@@ -7,7 +7,6 @@ import (
 	"io"
 
 	"github.com/goccy/go-json"
-	yaml "gopkg.in/yaml.v3"
 )
 
 type jsonEncoder struct {
@@ -38,9 +37,11 @@ func (je *jsonEncoder) PrintLeadingContent(writer io.Writer, content string) err
 	return nil
 }
 
-func (je *jsonEncoder) Encode(writer io.Writer, node *yaml.Node) error {
+func (je *jsonEncoder) Encode(writer io.Writer, node *CandidateNode) error {
+	log.Debugf("I need to encode %v", NodeToString(node))
+	log.Debugf("kids %v", len(node.Content))
 
-	if node.Kind == yaml.ScalarNode && je.UnwrapScalar {
+	if node.Kind == ScalarNode && je.UnwrapScalar {
 		return writeString(writer, node.Value+"\n")
 	}
 
@@ -54,14 +55,7 @@ func (je *jsonEncoder) Encode(writer io.Writer, node *yaml.Node) error {
 	encoder.SetEscapeHTML(false) // do not escape html chars e.g. &, <, >
 	encoder.SetIndent("", je.indentString)
 
-	var dataBucket orderedMap
-	// firstly, convert all map keys to strings
-	mapKeysToStrings(node)
-	errorDecoding := node.Decode(&dataBucket)
-	if errorDecoding != nil {
-		return errorDecoding
-	}
-	err := encoder.Encode(dataBucket)
+	err := encoder.Encode(node)
 	if err != nil {
 		return err
 	}
