@@ -4,8 +4,6 @@ import (
 	"fmt"
 	"io"
 	"net/url"
-
-	yaml "gopkg.in/yaml.v3"
 )
 
 type uriEncoder struct {
@@ -27,11 +25,10 @@ func (e *uriEncoder) PrintLeadingContent(writer io.Writer, content string) error
 	return nil
 }
 
-func (e *uriEncoder) Encode(writer io.Writer, originalNode *yaml.Node) error {
-	node := unwrapDoc(originalNode)
-	if guessTagFromCustomType(node) != "!!str" {
+func (e *uriEncoder) Encode(writer io.Writer, node *CandidateNode) error {
+	if node.guessTagFromCustomType() != "!!str" {
 		return fmt.Errorf("cannot encode %v as URI, can only operate on strings. Please first pipe through another encoding operator to convert the value to a string", node.Tag)
 	}
-	_, err := writer.Write([]byte(url.QueryEscape(originalNode.Value)))
+	_, err := writer.Write([]byte(url.QueryEscape(node.Value)))
 	return err
 }
