@@ -7,7 +7,6 @@ import (
 	"strings"
 
 	parse "github.com/a8m/envsubst/parse"
-	yaml "gopkg.in/yaml.v3"
 )
 
 type envOpPreferences struct {
@@ -18,39 +17,40 @@ type envOpPreferences struct {
 }
 
 func envOperator(d *dataTreeNavigator, context Context, expressionNode *ExpressionNode) (Context, error) {
-	envName := expressionNode.Operation.CandidateNode.Node.Value
+	envName := expressionNode.Operation.CandidateNode.Value
 	log.Debug("EnvOperator, env name:", envName)
 
 	rawValue := os.Getenv(envName)
 
 	preferences := expressionNode.Operation.Preferences.(envOpPreferences)
 
-	var node *yaml.Node
+	var node *CandidateNode
 	if preferences.StringValue {
-		node = &yaml.Node{
-			Kind:  yaml.ScalarNode,
+		node = &CandidateNode{
+			Kind:  ScalarNode,
 			Tag:   "!!str",
 			Value: rawValue,
 		}
 	} else if rawValue == "" {
-		return Context{}, fmt.Errorf("Value for env variable '%v' not provided in env()", envName)
+		return Context{}, fmt.Errorf("value for env variable '%v' not provided in env()", envName)
 	} else {
-		var dataBucket yaml.Node
-		decoder := yaml.NewDecoder(strings.NewReader(rawValue))
-		errorReading := decoder.Decode(&dataBucket)
-		if errorReading != nil {
-			return Context{}, errorReading
+		decoder := NewYamlDecoder(ConfiguredYamlPreferences)
+		if err := decoder.Init(strings.NewReader(rawValue)); err != nil {
+			return Context{}, err
 		}
-		//first node is a doc
-		node = unwrapDoc(&dataBucket)
+		var err error
+		node, err = decoder.Decode()
+
+		if err != nil {
+			return Context{}, err
+		}
+
 	}
 	log.Debug("ENV tag", node.Tag)
 	log.Debug("ENV value", node.Value)
 	log.Debug("ENV Kind", node.Kind)
 
-	target := &CandidateNode{Node: node}
-
-	return context.SingleChildContext(target), nil
+	return context.SingleChildContext(node), nil
 }
 
 func envsubstOperator(d *dataTreeNavigator, context Context, expressionNode *ExpressionNode) (Context, error) {
@@ -70,8 +70,7 @@ func envsubstOperator(d *dataTreeNavigator, context Context, expressionNode *Exp
 	}
 
 	for el := context.MatchingNodes.Front(); el != nil; el = el.Next() {
-		candidate := el.Value.(*CandidateNode)
-		node := unwrapDoc(candidate.Node)
+		node := el.Value.(*CandidateNode)
 		if node.Tag != "!!str" {
 			log.Warning("EnvSubstOperator, env name:", node.Tag, node.Value)
 			return Context{}, fmt.Errorf("cannot substitute with %v, can only substitute strings. Hint: Most often you'll want to use '|=' over '=' for this operation", node.Tag)
@@ -81,8 +80,7 @@ func envsubstOperator(d *dataTreeNavigator, context Context, expressionNode *Exp
 		if err != nil {
 			return Context{}, err
 		}
-		targetNode := &yaml.Node{Kind: yaml.ScalarNode, Value: value, Tag: "!!str"}
-		result := candidate.CreateReplacement(targetNode)
+		result := node.CreateReplacement(ScalarNode, "!!str", value)
 		results.PushBack(result)
 	}
 
