@@ -1,7 +1,5 @@
 package yqlib
 
-import "gopkg.in/yaml.v3"
-
 func equalsOperator(d *dataTreeNavigator, context Context, expressionNode *ExpressionNode) (Context, error) {
 	log.Debugf("-- equalsOperation")
 	return crossFunction(d, context, expressionNode, isEquals(false), true)
@@ -12,33 +10,29 @@ func isEquals(flip bool) func(d *dataTreeNavigator, context Context, lhs *Candid
 		value := false
 		log.Debugf("-- isEquals cross function")
 		if lhs == nil && rhs == nil {
+			log.Debugf("-- both are nil")
 			owner := &CandidateNode{}
 			return createBooleanCandidate(owner, !flip), nil
 		} else if lhs == nil {
 			log.Debugf("lhs nil, but rhs is not")
-			rhsNode := unwrapDoc(rhs.Node)
-			value := rhsNode.Tag == "!!null"
+			value := rhs.Tag == "!!null"
 			if flip {
 				value = !value
 			}
 			return createBooleanCandidate(rhs, value), nil
 		} else if rhs == nil {
 			log.Debugf("lhs not nil, but rhs is")
-			lhsNode := unwrapDoc(lhs.Node)
-			value := lhsNode.Tag == "!!null"
+			value := lhs.Tag == "!!null"
 			if flip {
 				value = !value
 			}
 			return createBooleanCandidate(lhs, value), nil
 		}
 
-		lhsNode := unwrapDoc(lhs.Node)
-		rhsNode := unwrapDoc(rhs.Node)
-
-		if lhsNode.Tag == "!!null" {
-			value = (rhsNode.Tag == "!!null")
-		} else if lhsNode.Kind == yaml.ScalarNode && rhsNode.Kind == yaml.ScalarNode {
-			value = matchKey(lhsNode.Value, rhsNode.Value)
+		if lhs.Tag == "!!null" {
+			value = (rhs.Tag == "!!null")
+		} else if lhs.Kind == ScalarNode && rhs.Kind == ScalarNode {
+			value = matchKey(lhs.Value, rhs.Value)
 		}
 		log.Debugf("%v == %v ? %v", NodeToString(lhs), NodeToString(rhs), value)
 		if flip {
